** extract male data from sashelp.heart **;
** add subject numbers using _N_ **;
** calculate BMI **;
** save subject count into &n **;
DATA heart_data (KEEP = subject
						agechddiag ageatstart ageatdeath deathcause
						height weight bmi weight_status
						diastolic systolic bp_status
						cholesterol chol_status
						);
	LENGTH subject 8;
	SET sashelp.heart (WHERE = (sex = 'Male')) END = eof;
	subject = _N_;
	bmi = ((weight / 2.2) / ((height * 2.54 / 100) ** 2));
	IF eof THEN CALL SYMPUT('n', STRIP(PUT(_N_, 8.)));
RUN;

** summarise sashelp.bmimen to get mean and SD **;
PROC SUMMARY DATA = sashelp.bmimen NWAY MISSING;
	CLASS age;
	VAR bmi;
	OUTPUT OUT = bmimen_stats MEAN = bmi_mean STDDEV = bmi_sd;
RUN;

** generate text for summary box in top-left of graph **;
DATA heart_text (KEEP = subject textgroup text x_text y_text);
	SET heart_data;
	BY subject;
	LENGTH textgroup $30
		   text $100
		   ;
	x = 0;
	y = 100;
	lineskip = -3;
	** Age, if non-missing **;
	IF NMISS(ageatstart) = 0 THEN DO;
		textgroup = 'Start';
		x_text = x;
		y_text = y;
		text = 'Age at start: ' || STRIP(PUT(ageatstart, 8.));
		OUTPUT;
		y + lineskip;
	END;
	** Age at CHD diagnosis, if non-missing **;
	IF NMISS(agechddiag) = 0 THEN DO;
		textgroup = 'CHD';
		x_text = x;
		y_text = y;
		text = 'Age at CHD diagnosis: '
		|| STRIP(PUT(agechddiag, 8.));
		OUTPUT;
		y + lineskip;
	END;
	** Age at death and cause, if non-missing **;
	IF NMISS(ageatdeath) = 0 THEN DO;
		textgroup = 'Death';
		x_text = x;
		y_text = y;
		text = 'Age at death: ' || STRIP(PUT(ageatdeath, 8.));
		IF CMISS(deathcause) = 0 THEN DO;
			text = STRIP(text)
				   || ', Cause: '
				   || STRIP(deathcause)
				   ;
		END;
		OUTPUT;
		y + lineskip;
	END;
	** Weight in kg and weight status, if both non-missing **;
	IF CMISS(weight, weight_status) = 0 THEN DO;
		textgroup = 'Weight';
		x_text = x;
		y_text = y;
		text = 'Weight at start: '
			   || STRIP(PUT(weight / 2.2, 8.1))
			   || ' kg, Status: '
			   || STRIP(weight_status)
			   ;
		OUTPUT;
		y + lineskip;
	END;
	** Cholesterol in mmol/L and cholesterol status,
	if both non-missing **;
	IF CMISS(cholesterol, chol_status) = 0 THEN DO;
		textgroup = 'Cholesterol';
		x_text = x;
		y_text = y;
		text = 'Cholesterol at start: '
			   || STRIP(PUT(cholesterol / 38.6598, 8.1))
			   || ' mmol/L, Status: '
			   || STRIP(chol_status)
			   ;
		OUTPUT;
		y + lineskip;
	END;
	24
	** Blood pressure and blood pressure status,
	if all non-missing **;
	IF CMISS(diastolic, systolic, bp_status) = 0 THEN DO;
		textgroup = 'BP';
		x_text = x;
		y_text = y;
		text = 'BP at start: '
			   || STRIP(PUT(systolic, BEST.))
			   || '/'
			   || STRIP(PUT(diastolic, BEST.))
			   || ', Status: '
			   || STRIP(bp_status)
			   ;
		OUTPUT;
		y + lineskip;
	END;
RUN;

** calculate BMI, BMI +- SD and BMI +- 2*SD lines **;
DATA bmimen_data (KEEP = subject
						 age bmi_mean
						 bmi_2sdplus bmi_sdplus bmi_sdless bmi_2sdless
						 );
	SET bmimen_stats;
	bmi_2sdplus = bmi_mean + (bmi_sd * 2);
	bmi_sdplus = bmi_mean + bmi_sd;
	bmi_sdless = bmi_mean - bmi_sd;
	bmi_2sdless = bmi_mean - (bmi_sd * 2);
	** include copy of lines for each subject **;
	DO subject = 1 TO &n.;
		OUTPUT;
	END;
RUN;

PROC SORT DATA = bmimen_data;
	BY subject age;
RUN;

** combine heart data, text box and BMI lines **;
DATA sasuser.plot_data;
	LENGTH section $10;
	SET heart_data (IN = a)
		heart_text (IN = b)
		bmimen_data (IN = c)
		;
	BY subject;
	SELECT;
		WHEN (a) section = 'Health';
		WHEN (b) section = 'Text';
		WHEN (c) section = 'BMI lines';
		OTHERWISE;
	END;
	LABEL subject = 'Subject'
		  age = 'Age'
		  bmi = 'BMI'
		  ;
RUN;

** create text colour map **;
DATA plot_attrmap (KEEP = id value textcolor);
	LENGTH value $30
		   id textcolor $50
		   ;
	id = 'colors';
	textcolor = 'blue';
	value = 'CHD';
	OUTPUT;
	textcolor = 'red';
	value = 'Death';
	OUTPUT;
	textcolor = 'black';
	value = 'Start';
	OUTPUT;
	value = 'Weight';
	OUTPUT;
	value = 'Cholesterol';
	OUTPUT;
	value = 'BP';
	OUTPUT;
RUN;

** Generate plot per subject of BMI lines,
subject BMI at start,
droplines to mark CHD diagnosis and death,
and top-left text box **;
OPTIONS NOBYLINE;
TITLE "Subject: #BYVAL(subject)";

PROC SGPLOT DATA = sasuser.plot_data DATTRMAP = plot_attrmap;
	WHERE subject IN (12);
	BY subject;
	** BMI lines **;
	LOESS X = age Y = bmi_2sdplus
		/ NOMARKERS LINEATTRS = (COLOR = DARKCYAN PATTERN = DASH);
	LOESS X = age Y = bmi_sdplus
		/ NOMARKERS LINEATTRS = (COLOR = DARKCYAN PATTERN = DASH);
	LOESS X = age Y = bmi_mean
		/ NOMARKERS LINEATTRS = (COLOR = DARKCYAN)
		  NAME = 'BMI' LEGENDLABEL = 'Mean BMI of population +/- SD';
	LOESS X = age Y = bmi_sdless
		/ NOMARKERS LINEATTRS = (COLOR = DARKCYAN PATTERN = DASH);
	LOESS X = age Y = bmi_2sdless
		/ NOMARKERS LINEATTRS = (COLOR = DARKCYAN PATTERN = DASH);
	** subject BMI it start **;
	SCATTER X = ageatstart Y = bmi
			/ MARKERATTRS = (COLOR = BLACK SYMBOL = SQUAREFILLED)
			  NAME = 'Start' LEGENDLABEL = 'Start';
	** CHD diagnosis and death markers **;
	DROPLINE X = agechddiag Y = 42
			/ LINEATTRS = (COLOR = BLUE PATTERN = DASHDOTDOT)
			  NAME = 'CHD Diag';
	DROPLINE X = ageatdeath Y = 42
			/ LINEATTRS = (COLOR = RED PATTERN = SHORTDASHDOT)
			  NAME = 'Death';
	** text box - BOTTOMRIGHT is position of text relative
	to x_text and y_text **;
	TEXT X = x_text Y = y_text TEXT = text
		/ ATTRID = colors GROUP = textgroup TEXTATTRS = (SIZE = 7pt)
		  POSITION = BOTTOMRIGHT VCENTER = BBOX
	X2AXIS Y2AXIS;
	** legend **;
	KEYLEGEND 'Start' 'CHD Diag' 'Death' 'BMI';
	** primary axes **;
	XAXIS MIN = 0 MAX = 90 OFFSETMIN = 0 OFFSETMAX = 0
		  LABEL = 'Age at start (years)';
	YAXIS MIN = 0 MAX = 50 OFFSETMIN = 0 OFFSETMAX = 0 LABEL = 'BMI';
	** secondary axes, used to position the text box **;
	X2AXIS MIN = 0 VALUES = (0 TO 1000 BY 1000) DISPLAY = NONE
		   OFFSETMIN = 0.01 OFFSETMAX = 0;
	Y2AXIS MIN = 0 VALUES = (0 TO 100 BY 100) DISPLAY = NONE
		   OFFSETMAX = 0;
RUN;